/*
 * pollingidm.c
 * 
 * The BSD License:
 *
 * Copyright (c) 2008, makai
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution.
 * 3 .Neither the name of the author nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software 
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include <stdarg.h>
#include <getopt.h>
#include "libpasori_com_libusb_multi.h"

extern char *optarg;
extern int optind, opterr, optopt;

/* definitions */
#define DEFAULT_SCAN_INTERVAL   100  /* millisecond */
#define ERR_CANNOT_OPEN_PASORI  0x01
#define ERR_CANNOT_INIT_PASORI  0x02
#define ERR_NO_CARD_OR_ERROR    0x03

/* global variables */
static int g_debug;
static int g_adhoc;
static int g_interval;
static char *g_errMsg[] = {
    "",
    "cannot open pasori",
    "cannot init pasori",
    "no card or fatal error"
};

/* 
 * dump for debug
 */
void
dump_pasori(pasori *p) {
    if (g_debug) {
        fprintf(stderr, "p->dev->filename:%s\n", p->dev->filename);
        fprintf(stderr, "p->dev->devnum:%d\n", p->dev->devnum);
    }
}

/*
 * output IDm to stdout.
 */
void
reply_IDm(pasori *p, uint8 *IDm) {
    int size = 8;
    int i;

    fprintf(stdout, "%04d", p->dev->devnum);
    for (i = 0; i != size; i++) {
        fprintf(stdout, "%02X",IDm[i]);
    }
    fprintf(stdout, "\n");
    fflush(stdout);
}

static void
reply_errmsg(int errCode) {
    char emsg[32];
    sprintf(emsg, "E%03d:%-15.15s", errCode, g_errMsg[errCode]);
    fprintf(stdout, "%s\n", emsg);
}

static void
err_exit(int errCode) {
    reply_errmsg(errCode);
    exit(errCode);
}

static void
debug(char *fmt, ...) {
    va_list argp;

    if (g_debug) {
        va_start(argp, fmt);
        fprintf(stderr, "debug:");
        vfprintf(stderr, fmt, argp);
        fprintf(stderr, "\n");
        va_end(argp);
    }
    fflush(stderr);
}

/*
 * display usage to stderr.
 */
static void
usage(char *name, char *withmsg) {
    if (withmsg != NULL) {
        fprintf(stderr, "%s\n", withmsg);
    }
    fprintf(stderr, "usage:%s [options]\n", basename(name));
    fprintf(stderr, "options:\n");
    fprintf(stderr, " -a pasoriNo :scan just once.\n");
    fprintf(stderr, " -d          :display debug messages to stderr.\n");
    fprintf(stderr, " -h          :display usage.\n");
    fprintf(stderr, " -i msec     :set scan interval. default=100.\n");
    fprintf(stderr, " -u times    :uso scan.\n");

    exit(1);
}

/*
 * start here.
 */
int
main(int argc, char **argv) {
    char emsg[64];
    char buf[64];
    pasori **pp;
    pasori *adhoc;
    felica* f;
    int option;
    int ret;
    int n_pasori;
    int i;

    /* handling options */
    opterr = 0;
    g_debug = 0;
    g_adhoc = 0;
    g_interval = DEFAULT_SCAN_INTERVAL;

    while ( 1 ) {
        option = getopt(argc, argv, "a:dhi:u:");
        if (option == -1) {
            break;
        }

        switch(option) {
            case 'a':
                strncpy(buf, optarg, 1);
                g_adhoc = atoi(buf);
                debug("set option -a with %d.", g_adhoc);
                break;
            case 'd':
                g_debug = 1;
                debug("set option -d.");
                break;
            case 'h':
                usage(argv[0], NULL);
                break;
            case 'i':
                g_interval = atoi(optarg);
                if (g_interval <= 5) {
                    g_interval = 5;
                    debug("interval must less than 5.");
                }
                debug("set option -i with %d.", g_interval);
                break;
            case 'u':
                debug("set option -u with %d.", atoi(optarg));
                for (i = 0; i < atoi(optarg); i++) {
                    printf("00010214E4004E09590A\n");
                    fflush(stdout);
                    Sleep(2000);
                }
                exit(0);
            case '?':
                sprintf(emsg, "Unknown option '%c'", optopt);
                usage(argv[0], emsg);
                break;
        }
        optarg = NULL;
    }

    /* initialize pasori */
    debug("Call pasori_open_multi()");
    n_pasori = pasori_open_multi(&pp);
    debug(".... pasori_open_multi() return %d", n_pasori);

    if(n_pasori == 0) {
        /* no pasori or error */
        err_exit(ERR_CANNOT_OPEN_PASORI);
    }

    if (g_adhoc) {
        /* scan just once */
        if (g_adhoc <= n_pasori) {
            adhoc = *(pp + (g_adhoc - 1));
            debug("Call pasori_init():%d", g_adhoc - 1);
            ret = pasori_init(adhoc);
            debug(".... pasori_init() return %d", ret);
        } else {
            pasori_close_multi(n_pasori, pp);
            err_exit(ERR_CANNOT_INIT_PASORI);
        }
        debug("Call felica_polling():%x", adhoc);
        f = (felica*)felica_polling(adhoc,POLLING_ANY,0,0);
        debug(".... felica_polling() return %x", f);
        if (f) {
            reply_IDm(adhoc, f->IDm);
            free(f);
        } else {
            pasori_close_multi(n_pasori, pp);
            err_exit(ERR_NO_CARD_OR_ERROR);
        }
    } else {
        /* loop forever */

        for (i = 0;i < n_pasori; i++) {
            debug("Call pasori_init():%d", i);
            ret = pasori_init(*(pp + i));
            debug(".... pasori_init() return %d", ret);
        }

        while ( 1 ) {
            for (i = 0; i < n_pasori; i++) {
                debug("Call felica_polling():%x", pp[i]);
                f = (felica*)felica_polling(pp[i],POLLING_ANY,0,0);
                debug(".... felica_polling() return %x", f);
                if (f) {
                    dump_pasori(pp[i]);
                    reply_IDm(pp[i], f->IDm);
                    free(f);
                    Sleep(500);
                }
            }
            Sleep(g_interval);
        }
    }

    if (pp != NULL) {
        pasori_close_multi(n_pasori, pp);
    }

    return 0;
}

/* EOF */
